function [GLM] = GLMfields(GLMset)
% =============================================================================
%      Structure describing Multivariate Generalized Linear Model (MGLM)
% =============================================================================
% CALLING: [GLM] = GLMfields(GLMset)
% -----------------------------------------------------------------------------
% GLMset ... settings of GLM
%       .dist ... distribution of response variable
%                 ('gamma', 'normal', 'poisson', 'binomial')
%       .link ... link function ('-1/mu','identity','ln(mu)','canonical')
% GLM ... structure describing GLM:
%       .b_theta ... expression for function b(theta) in terms of natural
%                    parameter theta identifying the type of distribution,
%                    e.g. for Gamma distributed 'y' we have b_theta = '-log(-theta)'
%       .b1_theta... expression for 1-st derivative mu(theta)=b'(theta)
%       .b2_theta ... expression for 2-nd derivative b''(theta)
%                       [optional, default by diff]
%       .b1_mu ... expression for inverse function of the 1-st
%                  derivative b'(theta) evaluated in mu: theta=b'inv(mu)
%       .g_mu   ... expression for the link function g(mu)
%       .g_eta ... expression for inverse function of the link
%                  function g(mu) evaluated in eta: mu=g_inv(eta)
%       .g1_mu ... expression for the 1-st derivative g'(mu)
%                          [optional, default by diff]
%       .u2_eta ... expression for 2-nd derivative u''(eta), u = inv(b') o inv(g)
%       .Phi ... dispersion Phi depending on distribution
%       .y   ... vector of observations of length n
%       .w   ... vector of (known) weights for y of length n, all(w)>0
%                  (diagonal matrix Delta=diag(GLM.w)/GLM.Phi)
%       .Z   ... the linear predictor matrix of size n x m
%       .theta_dom ... specifies interval domain for theta:
%                 = [a,b], -Inf <= a<=theta<=b <= Inf
%                 (optional field: default=[-Inf,Inf])
%       .mu_dom    ... specifies interval domain for mu:
%                 = [a,b], -Inf <= a<=mu<=b <= Inf
%                 (optional field: default=[-Inf,Inf])
% -------------------------------------------------------------------------
% (c) Zuzana Hrdlickova, Vitezslav Vesely, 2007
% 2.10.2007

switch GLMset.dist
    case 'gamma'
        GLM.b_theta  = '-log(-theta)';
        GLM.b1_theta = '- theta.^(-1)';
        GLM.b2_theta = 'theta.^(-2)';
        GLM.b1_mu = '-mu.^(-1)';
%         GLM.Phi = 1/alpha;       
        switch GLMset.link
            case 'identity'
                GLM.g_mu = 'mu';
                GLM.g_eta = 'eta';      
                GLM.g1_mu = '1';      
                GLM.u2_eta = '-2*(eta).^(-3)';
                GLM.mu_dom = [1e-4,Inf];
                GLM.theta_dom = [-Inf,-1e-4];                                                
            case 'log'
                GLM.g_mu = 'log(mu)';
                GLM.g_eta = 'exp(eta)';      
                GLM.g1_mu = 'mu.^(-1)';      
                GLM.u2_eta = '-(exp(eta)).^(-1)';
                GLM.mu_dom = [1e-3,Inf];
                GLM.theta_dom = [-Inf,-1e-4];                                                
            case 'canonical'
                GLM.g_mu = GLM.b1_mu;
                GLM.g_eta = regexprep(GLM.b1_theta,'theta', 'eta');
                GLM.g1_mu = regexprep(GLM.b2_theta,'theta', 'mu');
                GLM.u2_eta = 'abs(eps.^0)-1'; % we omitt entering 0 directly due to later eval usage 
                GLM.mu_dom = [1e-3,Inf];
                GLM.theta_dom = [-Inf,-1e-4];                                
        end;
    case 'poisson'
        GLM.b_theta  = 'exp(theta)';
        GLM.b1_theta = 'exp(theta)';
        GLM.b2_theta = 'exp(theta)';
        GLM.b1_mu = 'log(mu)';
        GLM.Phi = 1;
    case 'binomial'     % relative frequency
        GLM.b_theta  = 'log(1+exp(theta))';
        GLM.b1_theta = 'exp(theta)./(1+exp(theta))';
        GLM.b2_theta = 'exp(theta)./((1+exp(theta)).^2)';
        GLM.b1_mu = 'log(mu./(1-mu))';
%        GLM.Phi = 1/N;        
    case 'negbinomial'
        GLM.b_theta  = ['-',num2str(GLMset.k),'*log(1-exp(theta))'];
        GLM.b1_theta = [num2str(GLMset.k),'*exp(theta)./(1-exp(theta))'];
        GLM.b2_theta = [num2str(GLMset.k),'*exp(theta)./((1-exp(theta)).^2)'];
        GLM.b1_mu = ['log(mu./(',num2str(GLMset.k),'+mu))'];        
    case 'normal'
        GLM.b_theta  = ['0.5*theta',char(39),'*theta'];
        GLM.b1_theta = 'theta';
        GLM.b2_theta = 'eye(length(theta))';
        GLM.b1_mu = 'mu';
%        GLM.Phi = sigma2;        
    case 'multinomial' % relative frequency
        GLM.b_theta  = 'repmat(log(1+ones(1,size(theta,1))*exp(theta)),size(theta,1),1)';
        GLM.b1_theta = 'exp(theta)./repmat(1+ones(1,size(theta,1))*exp(theta),size(theta,1),1)';
        GLM.b2_theta = ['diag(exp(theta))/(1+ones(1,size(theta,1))*exp(theta))-(exp(theta)*exp(theta)',...
            char(39),')/((1+ones(1,size(theta,1))*exp(theta))^2)'];
         GLM.b1_mu = 'log(mu./repmat(1-ones(1,size(mu,1))*mu,size(mu,1),1))';  
%         GLM.Phi = 1/N;   
    otherwise
        error('Unknown distribution')
end;
if isequal(GLMset.link,'canonical')
                GLM.g_mu = GLM.b1_mu;
                GLM.g_eta = regexprep(GLM.b1_theta,'theta', 'eta');
                GLM.g1_mu = regexprep(GLM.b2_theta,'theta', 'mu');
                GLM.u2_eta = 'abs(eta.^0)-1'; % we omitt entering 0 directly due to later eval usage 
end;
                
