function [h,power_D,power_S] = plotpower(dist,id_param,param,v,h,ss,nuis,alpha)
% =============================================================================
%      Plot of asymptotic approximation of power of test of
%                   equality of expected values in
%    balanced one-way MANOVA type multivariate generalized linear model
%               based on Deviance and Score statistics
% =============================================================================
% CALLING: [h,power_D,power_S] = plotpower(dist,id_param,param,v,h,ss,nuis,alpha);
% -----------------------------------------------------------------------------
% INPUT
% dist ... distribution of respose variables
%  ('gamma', 'normal', 'poisson', 'binomial', 'multinomial', 'negbinomial')
% id_param ... = 1 parameters param, v, h correspond to parameters of
%                   appropriate balanced one-way MANOVA type MGLM:
%               theta_i = beta_0 + beta_i, i = 1,...,g, beta_1 = 0
%              = 0 parameters describe expected values of response
%              variables in balanced one-way MANOVA type MGLM
% param ... value of parameters/expected values under the null hypothesis
%           for multinomial q x g matrix (rows = classes of the
%           distribution, columns = groups)
% v, h ... parameters defining considered alternatives, so that for i-th
%           alternative:      param_i = param + v * h(i)
% ss ... size of each group in balanced one-way MANOVA type MGLM
% nuis ... nuisance parameter (default = 1), i.e.
%       (sigma^2 - 'normal', alpha - 'gamma', N - 'binomial', N -
%       'multinomial', k 'negbinomial')
% alpha ... significance level (default = 0.05)
%
% OUTPUT
% h ... see description in INPUT
% power.D ... asymptotic approximation of power of test based on Deviance in one-way anova type MGLM
% power.S ... asymptotic approximation of power of test based on Score statistics in one-way anova type MGLM
%
% EXAMPLES:
% 1) Model with gamma distribution
%    dist = 'gamma'; id_param = 1; param = [0.5,0,0,0]; v = 0:3;
%    h = linspace(0,0.4); ss = 20; nuis = 2;
% 2) Model with gamma distribution
%    dist = 'gamma'; id_param = 0; param = 3*ones(4,1); v = 0:3;
%    h = linspace(0,10); ss = 10;
% 3) Model with poisson distribution
%    dist = 'poisson'; id_param = 0; param = 2*ones(4,1); v = 0:3;
%    h = linspace(0,1); ss = 20;
% 4) Model with binomial distribution relative frequencies
%    dist = 'binomial'; id_param = 0; param = 0.3*ones(4,1); v = 0:3;
%    h = linspace(0,0.2); ss = 10;
% 5) Model with binomial distribution
%    dist = 'binomial'; id_param = 0; param = 3*ones(4,1); v = 0:3;
%    h = linspace(0,2); ss = 10; nuis = 10
% 5) Model with negative binomial distribution
%    dist = 'negbinomial'; id_param = 0; param = 3*ones(4,1); v = 0:3;
%    h = linspace(0,3); ss = 10;
% 6) Model with multinomial distribution
%    dist = 'multinomial'; id_param = 1;  h = linspace(0,1.5); ss = 10;
%    param = [0.1,0.2,0.1,0.2;0,0,0,0;0,0,0,0]';
%    v = [0,0,0,0;2,1,1,1;1,1,1,2]';
%
% More details in Hrdlickova,Z. Approximation of powers of some tests in one-way MANOVA
% type multivariate generalized linear model. Computational Statistics and
% Data Analysis. submitted. 2007.
%
% (c) Zuzana Hrdlickova, 2007
% 2.10.2007



%%% Initialization of parameters
if nargin == 7
    alpha = 0.05;
end;
if nargin == 6 
    alpha = 0.05;
    nuis = 1;
end;
if isempty(nuis)
    nuis = 1;
end;

%%% Input check
if length(param(:))~=length(v(:))
    error('Inputs param and v do not have the same number of components')
end;
if (length(ss)~= 1 || any(ss<0))
    error('Input ss is not a positive number')
end;
if (length(nuis)~= 1 || any(nuis<0))
    error('Input nuis is not a positive number')
end;
if (alpha< 0 || alpha >1)
    error('Input alpha is not a number in (0,1)')
end;

if id_param == 0
    switch dist
        case {'gamma','poisson','negbinomial'}
            if any(param <0)
                error('Negative expected values of chosen distribution')
            end;
        case {'binomial'}
            if all(round(param) == param)   % recalculate relative frequencies
                param = param/nuis;
                v = v/nuis;
            end;
            if any(param <0)|| any(param>1)
                error('Improper expected values of binomial distribution')
            end;
        case {'multinomial'}
            param = param/nuis;
            v = v/nuis;
            if any(param <0)|| any(sum(param)>1)
                error('Improper expected values of multinomial distribution')
            end;            
    end
    if  (~isequal(dist,'multinomial') && any(repmat(mean(param),size(param,1),1)~= param)) ||...
        (isequal(dist,'multinomial') && any(repmat(mean(param,2),1,size(param,2))~= param)) 
        error('Expected values are equal under the null hypothesis.')
    end;
end

if isequal(dist,'negbinomial')
    GLMset.k = nuis;
end;

GLMset.dist = dist;
GLMset.link = 'canonical';
GLM = GLMfields(GLMset);
switch dist
    case {'gamma','binomial'}
        GLM.Phi = 1/nuis;
    case {'poisson','negbinomial','multinomial'}
        GLM.Phi = 1;
    case 'normal'
        GLM.Phi = nuis;
end;


if ~isequal(dist, 'multinomial')
    q = 1;
    g = length(param);
else
    q = size(param,1);
    g = size(param,2);
    ss = ss*nuis;
end;

Z = eye(g);    % design matrix
Z (:,1) = ones(g,1);
Z  = kron(Z, eye(q));

v = v(:);
h = h(:);

switch id_param
    case 1
        param = param(:);
        beta_H = param;
        beta_A = repmat(beta_H,1,length(h))+ v * h';
    case 0
        if isequal(dist, 'multinomial')
            mu = param;
            eta = eval(GLM.g_mu);
            beta_H = pinv(Z)*eta(:);
            mu = repmat(mu,1,length(h))+ reshape(v * h',q,length(h)*g);
            eta = eval(GLM.g_mu);
            beta_A = pinv(Z)*reshape(eta(:),q*g,length(eta(:))/(q*g));
        else
            mu = param(:);
            beta_H = pinv(Z)*eval(GLM.g_mu);
            mu = repmat(mu,1,length(h))+ v * h';
            beta_A = pinv(Z)*eval(GLM.g_mu);
        end;
    otherwise
        error('Identifier of the type of parameter can be either 0 or 1')
end

[power_D,power_S] = powerDS(Z,beta_H,beta_A,q,ss,GLM,alpha);
figure
plot(h,power_D,h,power_S,'r')      % power of test based on Score statistics
title(['One-way MANOVA type MGLM with distribution: ',dist])
legend('Power of test based on deviance','Power of test based on score statistics','Location','Best')

