function [betaD,betaS] = powerDS(Z,beta,beta1,q,N,GLM,alpha)
% =============================================================================
% Asymptotic approximation of power of test of equality of expected values in
%    balanced one-way MANOVA type multivariate generalized linear model
%               based on Deviance and Score statistics
% =============================================================================
% CALLING: [betaD,betaS] = powerDS(Z,beta,beta1,q,N,GLM,alpha);
% -----------------------------------------------------------------------------
% INPUT
% Z ... design matrix of MGLM
% beta ... vector of parameters under the null hypothesis
% beta1 ... vector of parameters under alternatives
% q ... dimension of response vector in MGLM
% N ... number of observations in classes 
% GLM ... structure describing GLM (see GLMfields.m)
% alpha ... significance level (default = 0.05)
%
% OUTPUT
% betaD ... asymptotic approximation of power of test based on Deviance 
% betaS ... asymptotic approximation of power of test based on Score statistics 
%
% More details in Hrdlickova,Z. Approximation of powers of some tests in one-way MANOVA
% type multivariate generalized linear model. Computational Statistics and
% Data Analysis. submitted. 2007.
%
% (c) Zuzana Hrdlickova, 2007
% 2.10.2007


%%% Input check
if any ([q,N]<0)
    error('Negative value of parameter q or N')
end;    
if size(Z,2)~=length(beta)
    error('Size of given design matrix does not match the length of vector parameter beta')
end;
if size(beta1,1)~=length(beta)
    error('Length of vector beta under alternative does not match the length of vector parameter beta')
end;
if mod(size(Z,1),q)~=0
    error('Dimension of response vector does not match the size of design matrix')
end;

%%% Initialization of parameters
if nargin == 6
    alpha = 0.05;
end;
g = size(Z,1)/q;                    % number of classes in one-way ANOVA
p = length(beta);                   % dimension of vector parameter of MGLM
if length(N) ~= 1                   % number of observations
    disp(['Sample size is understood to be ',num2str(N(1)),' since the balanced model is considered']) 
    N = N(1);
end;
na = size(beta1,2);                 % number of considered alternatives
chikv = chi2inv(1-alpha,p-q);          


%%% functions for asymptotic approximation of power
GLM.b_mu = regexprep(GLM.b_theta,'theta',['(',GLM.b1_mu,')']);
GLM.b2_mu = regexprep(GLM.b2_theta,'theta',['(',GLM.b1_mu,')']);
K = []; L1 = []; PNC1 = []; betaD = []; L2 = []; PNC2 = []; betaS = [];


for ia = 1:na                                       % for each alternative
    
    %%% parameter under given alternative
    Beta = beta1(:,ia);
    
    %%% expected value of response variables (probabilies) under given alterantive  
    theta = Z*Beta;                             % natural parameter
    theta = reshape(theta,q,g);                 % row - dimension of response variable, column - class
    mu_qg = eval(GLM.b1_theta);
    Mu = mu_qg*ones(g,1)/g;                        % average of expected values
    
    eta = []; b_theta=[]; sigma = [];
    for i = 1:g
        mu = mu_qg(:,i);
        eta = [eta,eval(GLM.g_mu)];
        b_theta = [b_theta,eval(GLM.b_mu)];
        sigma = blkdiag(sigma,eval(GLM.b2_mu)*GLM.Phi);
    end;
    mu = Mu;
     Eta = eval(GLM.g_mu);
    B_theta = eval(GLM.b_mu);
    Sigma = eval(GLM.b2_mu)*GLM.Phi;
    
    %%% approximation of power of the test based on the deviance
    A = (inv(sigma) - kron(ones(g),inv(Sigma))/g);
    a = GLM.Phi^(-1)*(eta(:)-repmat(Eta,g,1));
    b = pinv(A)*a;
    B = sqrtm(sigma);   
    K = [K;(2*N/GLM.Phi)*sum(sum(mu_qg.*eta - b_theta/q + repmat(-Mu.*Eta+ B_theta/q,1,g) )) - N*a'*b];    
    [C,Lambda,C2] = svd((B'*A*B+(B'*A*B)')/2);   
    L1 = [L1;diag(Lambda)'];   
    PNC1 = [PNC1;((C'*inv(B)*b).^2)'*N];
    betaD = [betaD;1-Cdfchi(L1(ia,:),ones(p,1),PNC1(ia,:),chikv-K(ia))];
    
    %%% approximation of power of the test based on score statistics
    M = (eye(g)-ones(g)/g);
    [C,Lambda] = eig(B'*kron(M,inv(Sigma))*B);
    L2 = [L2;diag(Lambda)'];
    PNC2 = [PNC2;N*((C'*inv(B)*mu_qg(:)).^2)'];
    betaS = [betaS;1-Cdfchi(L2(ia,:),ones(p,1),PNC2(ia,:),chikv)];
end;
