function [sss] = samplesize(dist,id_param,param,v,h,power,id_test,nuis,alpha)
% =============================================================================
%      Sufficient sample size for test of equality of expected values in
%    balanced one-way MANOVA type multivariate generalized linear model
%               based on Deviance and Score statistics
% =============================================================================
% CALLING: [sss] = samplesize(dist,id_param,param,v,h,power,nuis,alpha);
% -----------------------------------------------------------------------------
% INPUT
% dist ... distribution of respose variables
%  ('gamma', 'normal', 'poisson', 'binomial', 'multinomial', 'negbinomial')
% id_param ... = 1 parameters param, v, h correspond to parameters of
%                   appropriate balanced one-way MANOVA type MGLM
%              = 0 parameters describe expected values of response
%              variables in balanced one-way MANOVA type MGLM
% param ... value of parameters/expected values under the null hypothesis
%           for multinomial q x g matrix (rows = classes of the
%           distribution, columns = groups)
% v, h ... parameters defining considered alternatives, so that for i-th
%           alternative:      param_i = param + v * h(i)
% power ... assigned required power of the test
% id_test ... test based on deviance (=1) or score statistics (=2)
% nuis ... nuisance parameter (default = 1), i.e.
%       (sigma^2 - 'normal', alpha - 'gamma', N - 'binomial', N -
%       'multinomial', k 'negbinomial')
% alpha ... significance level (default = 0.05)
%
% OUTPUT
% sss ... sufficient sample size
%
% EXAMPLES:
% 1) Model with gamma distribution
%    dist = 'gamma'; id_param = 1; param = [5,0,0,0]; v = [0,1,2,3];
%    h = 0.8; power = 0.9; nuis = 5; id_test = 1;
% 2) Model with gamma distribution
%    dist = 'gamma'; id_param = 0; param = 3*ones(4,1); v = 0:3;
%    h = 0.8; power = 0.9; nuis = 1; id_test = 1;
% 3) Model with poisson distribution
%    dist = 'poisson'; id_param = 0; param = 5*ones(4,1); v = 0:3;
%    h = 1; power = 0.9; id_test = 1;
% 4) Model with binomial distribution
%    dist = 'binomial'; id_param = 0; param = 0.3*ones(4,1); v = 0:3;
%    h = 0.2; power = 0.9; id_test = 0; 
% 5) Model with negative binomial distribution
%    dist = 'negbinomial'; id_param = 0; param = 3*ones(4,1); v = 0:3;
%    h = 0.2; power = 0.9; id_test = 0;
% 6) Model with multinomial distribution
%    dist = 'multinomial'; id_param = 1;  h = 1; 
%    param = [0.1,0.2,0.1,0.2;0,0,0,0;0,0,0,0]';
%    v = [0,0,0,0;2,1,1,1;1,1,1,2]';  power = 0.9; id_test = 1; nuis = 1;
%
%
% More details in Hrdlickova,Z. Approximation of powers of some tests in one-way MANOVA
% type multivariate generalized linear model. Computational Statistics and
% Data Analysis. submitted. 2007.
%
% (c) Zuzana Hrdlickova, 2007
% 2.10.2007

%%% Initialization of parameters
if nargin == 8
    alpha = 0.05;
end;
if nargin == 7
    alpha = 0.05;
    nuis = 1;
end;
if isempty(nuis)
    nuis = 1;
end;

%%% Input check
if length(param(:))~=length(v(:))
    error('Inputs param and v do not have the same number of components')
end;
if (length(ss)~= 1 || any(ss<0))
    error('Input ss is not a positive number')
end;
if (length(nuis)~= 1 || any(nuis<0))
    error('Input nuis is not a positive number')
end;
if (alpha< 0 || alpha >1)
    error('Input alpha is not a number in (0,1)')
end;

if id_param == 0
    switch dist
        case {'gamma','poisson','negbinomial'}
            if any(param <0)
                error('Negative expected values of chosen distribution')
            end;
        case {'binomial'}
            if all(round(param) == param)   % recalculate relative frequencies
                param = param/nuis;
                v = v/nuis;
            end;
            if any(param <0)|| any(param>1)
                error('Improper expected values of binomial distribution')
            end;
        case {'multinomial'}
            param = param/nuis;
            v = v/nuis;
            if any(param <0)|| any(sum(param)>1)
                error('Improper expected values of multinomial distribution')
            end;            
    end
    if  (~isequal(dist,'multinomial') && any(repmat(mean(param),size(param,1),1)~= param)) ||...
        (isequal(dist,'multinomial') && any(repmat(mean(param,2),1,size(param,2))~= param)) 
        error('Expected values are equal under the null hypothesis.')
    end;
end

test = str2mat('deviance','score statistic');

switch dist
    case 'negbinomial'
        GLMset.k = nuis;
end;

GLMset.dist = dist;
GLMset.link = 'canonical';
GLM = GLMfields(GLMset);
switch dist
    case {'gamma','binomial'}
        GLM.Phi = 1/nuis;
    case {'poisson','negbinomial','multinomial'}
        GLM.Phi = 1;
    case 'normal'
        GLM.Phi = nuis;
end;

if ~isequal(dist, 'multinomial')
    q = 1;
    r = 1;
    g = length(param);
else
    q = size(param,1);
    r = q;
    g = size(param,2);
end;

Z = eye(g);    % design matrix
Z (:,1) = ones(g,1);
Z  = kron(Z, eye(q));

v = v(:);
h = h(:);
if id_param == 1
    param = param(:);
    beta_H = param;
    beta_A = repmat(beta_H,1,length(h))+ v * h';
else
    if isequal(dist, 'multinomial')
        param = param/nuis;
        v = v/nuis;
        mu = param;
        eta = eval(GLM.g_mu);
        beta_H = pinv(Z)*eta(:);
        mu = repmat(mu,1,length(h))+ reshape(v * h',q,length(h)*g);
        eta = eval(GLM.g_mu);
        beta_A = pinv(Z)*reshape(eta(:),q*g,length(eta(:))/(q*g));
    else
        mu = param(:);
        beta_H = pinv(Z)*eval(GLM.g_mu);
        mu = repmat(mu,1,length(h))+ v * h';
        beta_A = pinv(Z)*eval(GLM.g_mu);
    end;
end

%[output,fval,exitflag] = fsolve(@(N) cpower(N,power,id_test,Z,beta_H,beta_A,q,GLM,alpha),10);
[sss,fval,exitflag] = fsolve(@cpower,1,[],power,id_test,Z,beta_H,beta_A,q,GLM,alpha);
if isequal(dist, 'multinomial')
    sss = sss/nuis;
end;
disp(['One-way MANOVA type MGLM with distribution: ',dist])
disp(['Sufficient sample size for test based on ',deblank(test(id_test,:)),':  ',num2str(sss)])